#ifndef DOCTAB_H
#define DOCTAB_H

#include <QTextEdit>
#include <QtWidgets>
#include "highlighter.h"



class DocTab : public QPlainTextEdit
{
Q_OBJECT

private:    //fields

QString f_fDirectory;
QString f_fName;
QString f_fExtension;
bool f_changed;
Highlighter* f_highlighter;

private:    //methods

public:     //methods

DocTab(QWidget* p_parent = 0);

QString getFileDirectory() const                ///returns file directory
{return f_fDirectory;}

QString getFileName() const                     ///returns file name
{return f_fName;}

bool wasChanged() const                         ///returns bool representing if file was changed since loading
{return f_changed;}

QString getFileExt() const
{return f_fExtension;}

void setFileName(const QString& p_name)         ///sets file name for saving
{f_fName = p_name;}

signals:

void changeWidTitle(const QString&, const QString&);            ///signals about loaded file name
void fileChanded(const QString&);               ///signals about changes applied to the file

public slots:

QString sltLoad();                              ///shows FileDialog to choose loading file, gives control to sltLoad(const QString&)
QString sltLoad(const QString&);                ///loads file with the directory as a parameter
void sltIsCodeDoc();                            ///in case file has code file extension
void sltSave();                                 ///saves file or gives control to sltSaveAs if file directory is not defined (in case of new file made)
void sltSaveAs(const QString&);                 ///saves file with file name tip as a parameter
void sltLoadHighlight();                        ///loads highlight settings depending on extension, highlights code
void sltDocChanged();                           ///makes changes in f_changed if file was changed
void sltSetChanged(bool);                       ///set f_changed to parameter

///  Line numeration methods

void lineNumberAreaPaintEvent(QPaintEvent *event);
int lineNumberAreaWidth();

protected:

void resizeEvent(QResizeEvent *event) override;

///  Line numeration slots

private slots:

void updateLineNumberAreaWidth(int newBlockCount);
void highlightCurrentLine();
void updateLineNumberArea(const QRect &rect, int dy);

///  Line numeration field

private:
QWidget *lineNumberArea;


};

///  Line numeration widget class

class LineNumberArea : public QWidget
{
public:
LineNumberArea(DocTab *editor) : QWidget(editor), codeEditor(editor)
{}

QSize sizeHint() const override
{
  return QSize(codeEditor->lineNumberAreaWidth(), 0);
}

protected:
void paintEvent(QPaintEvent *event) override
{
  codeEditor->lineNumberAreaPaintEvent(event);
}

private:
DocTab *codeEditor;
};




#endif // DOCTAB_H
